import click
from sqlite3 import Error
import pprint
from .api_wrapper import tenb_connection
from .database import new_db_connection
import textwrap


tio = tenb_connection()


def find_by_plugin(pid):
    try:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT asset_ip, asset_uuid, fqdn, network from vulns LEFT JOIN assets ON asset_uuid = uuid where plugin_id=%s" % pid)

            rows = cur.fetchall()

            click.echo("\n{:8s} {:16s} {:46s} {:40s} {}".format("Plugin", "IP Address", "FQDN", "UUID", "Network UUID"))
            click.echo("-" * 150)
            for row in rows:
                click.echo("{:8s} {:16s} {:46s} {:40s} {}".format(str(pid), row[0], textwrap.shorten(row[2], 46), row[1], row[3]))

    except Error as e:
        click.echo(e)


@click.group(help="Discover what is in Tenable.io")
def find():
    pass


@find.command(help="Find Assets where a plugin fired")
@click.argument('plugin_id')
@click.option('--output', default='', help='Find Assets based on the text in the output')
def plugin(plugin_id, output):
    if not str.isdigit(plugin_id):
        click.echo("You didn't enter a number")
        exit()
    else:
        if output != "":
            try:
                plugin_db = r"navi.db"
                plugin_conn = new_db_connection(plugin_db)
                with plugin_conn:
                    click.echo("\n{:8s} {:16s} {:46s} {:40s} {}".format("Plugin", "IP Address", "FQDN", "UUID", "Network UUID"))
                    click.echo("-" * 150)
                    plugin_cur = plugin_conn.cursor()
                    plugin_cur.execute("SELECT asset_ip, asset_uuid, fqdn, network from vulns LEFT JOIN assets ON asset_uuid = uuid where plugin_id='" + plugin_id + "' and output LIKE '%" + output + "%';")
                    plugin_data = plugin_cur.fetchall()
                    for row in plugin_data:
                        click.echo("{:8s} {:16s} {:46s} {:40s} {}".format(str(plugin_id), row[0], textwrap.shorten(row[2], 46), row[1], row[3]))
            except Error:
                pass
        else:
            find_by_plugin(plugin_id)


@find.command(help="Find Assets where Text was found in the output of any plugin")
@click.argument('out_put')
def output(out_put):
    try:
        plugin_db = r"navi.db"
        plugin_conn = new_db_connection(plugin_db)
        with plugin_conn:
            click.echo("\n{:8s} {:16s} {:46s} {:40s} {}".format("Plugin", "IP Address", "FQDN", "UUID", "Network UUID"))
            click.echo("-" * 150)
            plugin_cur = plugin_conn.cursor()
            plugin_cur.execute("SELECT asset_ip, asset_uuid, fqdn, network, plugin_id from vulns LEFT JOIN assets ON asset_uuid = uuid where output LIKE '%" + str(out_put) + "%';")
            plugin_data = plugin_cur.fetchall()
            for row in plugin_data:
                click.echo("{:8s} {:16s} {:46s} {:40s} {}".format(row[4], row[0], textwrap.shorten(row[2], 46), row[1], row[3]))
    except Error:
        pass


@find.command(help="Find Docker Hosts using plugin 93561")
def docker():
    click.echo("Searching for RUNNING docker containers...")
    find_by_plugin(str(93561))


@find.command(help="Find Potential Web Apps using plugin 1442 and 22964")
def webapp():
    database = r"navi.db"
    conn = new_db_connection(database)
    print("\nPotential Web Applications Report\n")
    with conn:
        cur = conn.cursor()
        cur.execute("SELECT output, asset_uuid, asset_ip, network FROM vulns LEFT JOIN assets ON asset_uuid = uuid where plugin_id ='12053';")

        rows = cur.fetchall()

        for row in rows:
            host = row[0].split()
            final_host = host[3][:-1]
            uuid = row[1]
            print()
            print("*" * 50)
            print("Asset IP: {}".format(row[2]))
            print("Asset UUID: {}".format(row[1]))
            print("Network UUID: {}".format(row[3]))
            print("*" * 50)
            cur2 = conn.cursor()
            cur2.execute("SELECT output, port FROM vulns where plugin_id ='22964' and asset_uuid='{}';".format(uuid))
            new_row = cur2.fetchall()
            print("\nWeb Apps Found")
            print("-" * 14)
            for service in new_row:

                if "web" in service[0]:

                    if "through" in service[0]:
                        print("https://{}:{}".format(final_host, service[1]))
                    else:
                        print("http://{}:{}".format(final_host, service[1]))

            cur3 = conn.cursor()
            cur3.execute("SELECT output, port FROM vulns where plugin_id ='93561' and asset_uuid='{}';".format(uuid))
            doc_row = cur3.fetchall()
            if doc_row:
                print("\nThese web apps might be running on one or more of these containers:\n")
            for doc in doc_row:
                plug = doc[0].splitlines()
                for x in plug:
                    if "Image" in x:
                        print(x)
                    if "Port" in x:
                        print(x)
                        print()
            print("-" * 100)


@find.command(help="Find Assets with Credential Issues using plugin 104410")
def creds():
    click.echo("\nBelow are the Assets that have had Credential issues\n")
    find_by_plugin(104410)


@find.command(help="Find Assets where a plugin fired")
@click.argument('minute')
def scantime(minute):
    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:
        click.echo("\n*** Below are the assets that took longer than {} minutes to scan ***".format(str(minute)))
        cur = conn.cursor()
        cur.execute("SELECT * from vulns where plugin_id='19506';")

        data = cur.fetchall()
        try:
            click.echo("\n{:16s} {:40s} {:25s} {:25s} {}".format("Asset IP", "Asset UUID", "Started", "Finished", "Scan UUID"))
            click.echo("-" * 150)
            for vulns in data:

                output = vulns[6]

                # split the output by return
                parsed_output = output.split("\n")

                # grab the length so we can grab the seconds
                length = len(parsed_output)

                # grab the scan duration- second to the last variable
                duration = parsed_output[length - 2]

                # Split at the colon to grab the numerical value
                seconds = duration.split(" : ")

                # split to remove "secs"
                number = seconds[1].split(" ")

                # grab the number for our minute calculation
                final_number = number[0]

                # convert seconds into minutes
                minutes = int(final_number) / 60

                # grab assets that match the criteria
                if minutes > int(minute):
                    try:
                        click.echo("{:16s} {:40s} {:25s} {:25s} {}".format(str(vulns[1]), str(vulns[2]),
                                                                           str(vulns[14]), str(vulns[13]),
                                                                           str(vulns[15])))
                    except ValueError:
                        pass
            click.echo()
        except ValueError:
            pass


@find.command(help="Find Assets that have not been scanned in any Cloud")
def ghost():
    try:
        click.echo("\n{:11s} {:15s} {:45} {}".format("Source", "IP", "FQDN", "First seen"))
        click.echo("-" * 150)
        for assets in tio.workbenches.assets(("sources", "set-hasonly", "AWS")):
            for source in assets['sources']:
                if source['name'] == 'AWS':
                    aws_ip = assets['ipv4'][0]
                    try:
                        aws_fqdn = assets['fqdn'][0]
                    except IndexError:
                        aws_fqdn = assets['fqdn'][0]

                    click.echo("{:11s} {:15s} {:45} {}".format(str(source['name']), str(aws_ip),
                                                               str(aws_fqdn), source['first_seen']))
        click.echo()

        for gcp_assets in tio.workbenches.assets(("sources", "set-hasonly", "GCP")):
            for gcp_source in gcp_assets['sources']:
                if gcp_source['name'] == 'GCP':
                    gcp_ip = gcp_assets['ipv4'][0]
                    try:
                        gcp_fqdn = gcp_assets['fqdn'][0]
                    except IndexError:
                        gcp_fqdn = "NO FQDN FOUND"

                    click.echo("{:11s} {:15s} {:45} {}".format(gcp_source['name'], gcp_ip, gcp_fqdn,
                                                               gcp_source['first_seen']))
        click.echo()

        for az_assets in tio.workbenches.assets(("sources", "set-hasonly", "AZURE")):
            for az_source in az_assets['sources']:
                if az_source['name'] == 'AZURE':

                    az_ip = az_assets['ipv4'][0]
                    try:
                        az_fqdn = az_assets['fqdn'][0]
                    except IndexError:
                        az_fqdn = "NO FQDN Found"

                    click.echo("{:11s} {:15s} {:45} {}".format(az_source['name'], az_ip, az_fqdn,
                                                               az_source['first_seen']))
        click.echo()

    except Exception as E:
        click.echo("Check your API keys or your internet connection")
        click.echo(E)


@find.command(help="Find Assets with a given port open")
@click.argument('open_port')
def port(open_port):
    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:
        cur = conn.cursor()
        cur.execute("SELECT plugin_id, asset_ip, asset_uuid, fqdn, network from vulns LEFT JOIN assets ON asset_uuid = uuid where port=" + open_port + " and (plugin_id='11219' or plugin_id='14272' or plugin_id='14274' or plugin_id='34220' or plugin_id='10335');")

        data = cur.fetchall()

        try:
            click.echo("\nThe Following assets had Open ports found by various plugins")

            click.echo("\n{:8s} {:16s} {:46s} {:40s} {}".format("Plugin", "IP Address", "FQDN", "UUID", "Network UUID"))
            click.echo("-" * 150)

            for vulns in data:
                click.echo("{:8s} {:16s} {:46s} {:40s} {}".format(str(vulns[0]), vulns[1], textwrap.shorten(vulns[3], 46), vulns[2], vulns[4]))

            click.echo()
        except ValueError:
            pass


@find.command(help="Find Assets through a SQL query.")
@click.argument('statement')
def query(statement):
    database = r"navi.db"
    query_conn = new_db_connection(database)
    with query_conn:

        cur = query_conn.cursor()
        cur.execute(statement)

        data = cur.fetchall()
        pprint.pprint(data)


@find.command(help="Find Assets with a given name")
@click.argument('plugin_name')
def name(plugin_name):
    try:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT asset_ip, asset_uuid, plugin_name, plugin_id from vulns where plugin_name LIKE '%" + plugin_name + "%';")

            plugin_data = cur.fetchall()
            click.echo("\nThe Following assets had '{}' in the Plugin Name".format(plugin_name))
            click.echo("\n{:8s} {:20} {:45} {:70} ".format("Plugin", "IP address", "UUID", "Plugin Name"))
            click.echo("-" * 150)
            for vulns in plugin_data:
                click.echo("{:8s} {:20} {:45} {:70}".format(vulns[3], vulns[0], str(vulns[1]), textwrap.shorten(str(vulns[2]), 65)))
            click.echo()
    except Error:
        pass



import click
import time
from .api_wrapper import tenb_connection, navi_version
from .scanners import nessus_scanners
from .database import new_db_connection
from .error_msg import error_msg
from .licensed_count import get_licensed
from sqlite3 import Error
import textwrap

tio = tenb_connection()


@click.group(help="Display or Print information found in Tenable.io")
def display():
    pass


@display.command(help="List all of the scanners")
def scanners():
    nessus_scanners()


@display.command(help="List all of the Users")
def users():
    click.echo("\n{:34s} {:40s} {:40s} {:10s} {}".format("User Name", "Login Email", "UUID", "ID", "Enabled"))
    click.echo("-" * 150)
    for user in tio.users.list():
        click.echo("{:34s} {:40s} {:40s} {:10s} {}".format(str(user["name"]), str(user["username"]), str(user['uuid']), str(user['id']), str(user['enabled'])))
    click.echo()


@display.command(help="List all Exclusions")
def exclusions():
    for exclusion in tio.exclusions.list():
        click.echo("\n{} {}".format("Exclusion Name :", exclusion["name"]))
        click.echo("-" * 150)
        click.echo("{}".format(str(exclusion["members"])))
    click.echo()


@display.command(help="List all containers and their Vulnerability  Scores")
def containers():
    # Use CS module
    resp = tio.get('container-security/api/v2/images?limit=1000')
    data = resp.json()
    click.echo("{:35s} {:35s} {:15s} {:15s} {:10s}".format("Container Name", "Repository ID", "Tag", "Docker ID", "# of Vulns"))
    click.echo("-" * 150)
    try:
        for images in data["items"]:
            click.echo("{:35s} {:35s} {:15s} {:15s} {:10s}".format(images["name"], images["repoName"], images["tag"], str(images["imageHash"]), str(images["numberOfVulns"])))
    except KeyError:
        pass
    click.echo()


@display.command(help="List The actor and the action in the log file")
def logs():
    events = tio.audit_log.events()
    click.echo("{:24s} {:30s} {}".format("Event Date", "Action Taken", "User"))
    click.echo("-" * 150)
    for log in events:
        click.echo("{:24s} {:30s} {:30s}".format(str(log['received']), str(log['action']), str(log['actor']['name'])))
    click.echo()


@display.command(help="List the running Scans")
def running():
    click.echo("\n{:60s} {:10s} {:30s}".format("Scan Name", "Scan ID", "Status"))
    click.echo("-" * 150)
    for scan in tio.scans.list():
        if scan['status'] == "running":
            click.echo("{:60s} {:10s} {:30s}".format(str(scan['name']), str(scan['id']), str(scan['status'])))
    click.echo()


@display.command(help="List all Scans")
def scans():
    click.echo("\n{:60s} {:10s} {:30s} {}".format("Scan Name", "Scan ID", "Status", "UUID"))
    click.echo("-" * 150)
    for scan in tio.scans.list():
        click.echo("{:60s} {:10s} {:30s} {}".format(str(scan['name']), str(scan['id']), str(scan['status']), str(scan['uuid'])))
    click.echo()


@display.command(help="Nessus Network Monitor assets and their vulnerability scores")
def nnm():
    for scan in tio.scans.list():
        if str(scan["type"]) == "pvs":
            resp = tio.get('scans/{}'.format(scan["id"]))
            data = resp.json()
            click.echo("\n{:20} {}".format("IP Address", "Score"))
            click.echo("-" * 150)
            for host in data["hosts"]:
                click.echo("{:20} {}".format(str(host["hostname"]), str(host["score"])))
            click.echo()


@display.command(help="Assets found in the last 30 days")
def assets():
    click.echo("\nBelow are the assets found in the last 30 days")
    click.echo("\n{:36} {:65} {:15} {}".format("IP Address(es)", "FQDN(s)", "Exposure Score", "Sources"))
    click.echo("-" * 150)
    count = 0
    for asset in tio.workbenches.assets():
        count += 1
        sources = []
        for source in asset["sources"]:
            sources.append(source['name'])
        click.echo("{:36} {:65} {:15} {}".format(str(asset["ipv4"]), str(asset["fqdn"]), str(asset["exposure_score"]), sources))

    click.echo("\nTotal: {}".format(count))


@display.command(help="Scan Policies")
def policies():
    click.echo("\n{:40s} {:61s} {}".format("Policy Name", "Description", "Template ID"))
    click.echo("-" * 150)
    for policy in tio.policies.list():
        click.echo("{:40s} {:61s} {}".format(str(policy['name']), str(policy['description']), policy['template_uuid']))
    click.echo()


@display.command(help="List Connector Details and Status")
def connectors():
    resp = tio.get('settings/connectors')
    data = resp.json()
    click.echo("\n{:11s} {:40s} {:40s} {:30s} {}".format("Type", "Connector Name", "Connector ID", "Last Sync", "Schedule"))
    click.echo("-" * 150)
    for conn in data["connectors"]:
        schedule = str(conn['schedule']['value']) + " " + str(conn['schedule']['units'])
        try:
            last_sync = conn['last_sync_time']
        except KeyError:
            last_sync = "Hasn't synced"
        click.echo("{:11s} {:40s} {:40s} {:30s} {}".format(str(conn['type']), str(conn['name']), str(conn['id']), last_sync, schedule))
    click.echo()


@display.command(help="List Access Groups")
def agroups():
    rules = "Not Rule Based"
    try:
        click.echo("\n{:25s} {:40s} {:25} {}".format("Group Name", "Group ID", "Last Updated", "Rules"))
        click.echo("-" * 150)
        for group in tio.access_groups.list():
            try:
                updated = group['updated_at']
            except KeyError:
                updated = "Not Updated"

            details = tio.access_groups.details(group['id'])
            try:
                for rule in details['rules']:
                    rules = str(rule['terms'])
            except KeyError:
                rules = "Not Rule Based"
            click.echo("{:25s} {:40s} {:25} {:60s}".format(str(group['name']), str(group['id']), str(updated), textwrap.shorten(rules, width=60)))
        click.echo()
    except Exception as E:
        error_msg(E)


@display.command(help="Print T.io Status and Account info")
def status():
    try:
        data = tio.server.properties()
        session_data = tio.session.details()
        click.echo("\nTenable IO Information")
        click.echo("-" * 25)
        click.echo("{} {}".format("Container ID : ", session_data["container_id"]))
        click.echo("{} {}".format("Container UUID :", session_data["container_uuid"]))
        click.echo("{} {}".format("Container Name : ", session_data["container_name"]))
        click.echo("{} {}".format("Site ID :", data["analytics"]["site_id"]))
        click.echo("{} {}".format("Region : ", data["region"]))

        click.echo("\nLicense information")
        click.echo("-" * 25)
        click.echo("{} {}".format("Licensed Assets : ", get_licensed()))
        click.echo("{} {}".format("Agents Used : ", data['license']["agents"]))
        try:
            click.echo("{} {}".format("Expiration Date : ", data['license']["expiration_date"]))
        except:
            pass
        click.echo("{} {}".format("Scanners Used : ", data['license']["scanners"]))
        click.echo("{} {}".format("Users : ", data["license"]["users"]))

        click.echo("\nEnabled Apps")
        click.echo("-" * 15)
        click.echo()
        for key in data["license"]["apps"]:
            click.echo(key)
            click.echo("-" * 5)
            try:
                click.echo("{} {}".format("Expiration: ", str(data["license"]["apps"][key]["expiration_date"])))
            except KeyError:
                pass
            click.echo("{} {}".format("Mode: ", str(data["license"]["apps"][key]["mode"])))
            click.echo()

    except Exception as E:
        error_msg(E)


@display.command(help="Print Agent information")
def agents():
    click.echo("\n{:46s} {:20} {:20} {:20} {:10} {}".format("Agent Name", "IP Address", "Last Connect Time", "Last Scanned Time", "Status", "Groups"))
    click.echo("-" * 150)

    for agent in tio.agents.list():
        last_connect = agent['last_connect']
        last_connect_time = time.strftime("%b %d %H:%M:%S", time.localtime(last_connect))

        try:
            last_scanned = agent['last_scanned']
            last_scanned_time = time.strftime("%b %d %H:%M:%S", time.localtime(last_scanned))
        except KeyError:
            # I assume if we can't pull as scanned time, it doesn't exist
            last_scanned_time = "Not Scanned"
        groups = ''
        try:
            for group in agent['groups']:
                groups = groups + ", " + group['name']
        except KeyError:
            pass
        click.echo("{:46s} {:20s} {:20s} {:20s} {:10s} {}".format(str(agent['name']), str(agent['ip']),
                                                                  str(last_connect_time), str(last_scanned_time),
                                                                  str(agent['status']), str(groups[1:])))
    click.echo()


@display.command(help="Print Target Groups")
def tgroups():
    print()
    print("*" * 40)
    print("Target Groups are going to be retired use the Migration script to covert them to tags")
    print("https://github.com/packetchaos/tio_automation/blob/master/migrate_target_groups.py")
    print("*" * 40)
    print("\nTarget Group Name".ljust(41), "TG ID".ljust(10), "Owner".ljust(30), "Members")
    print("-" * 100)
    for targets in tio.target_groups.list():
        mem = targets['members']
        print(str(targets['name']).ljust(40), str(targets['id']).ljust(10), str(targets['owner']).ljust(30), textwrap.shorten(mem, width=60))
    print()


@display.command(help="Print Licensed Assets")
def licensed():
    click.echo("\n{} {}".format("Licensed Count: ", get_licensed()))
    click.echo()
    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:
        cur = conn.cursor()
        cur.execute("SELECT ip_address, fqdn, last_licensed_scan_date from assets where last_licensed_scan_date !=' ';")
        data = cur.fetchall()

        click.echo("{:20s} {:65s} {}".format("IP Address", "Full Qualified Domain Name", "Licensed Date"))
        click.echo("-" * 150)
        click.echo()
        count = 0
        for asset in data:
            count += 1
            ipv4 = asset[0]
            fqdn = asset[1]
            licensed_date = asset[2]
            # Don't display Web applications in this output
            if ipv4 != " ":
                click.echo("{:20s} {:65s} {}".format(str(ipv4), str(fqdn), licensed_date))
    click.echo("\nTotal: {}".format(count))


@display.command(help="Print Tag Information")
def tags():
    click.echo("\n{:30s} {:35s} {}".format("Category", "  Value", "  Value UUID"))
    click.echo("-" * 150)
    for tag_values in tio.tags.list():
        try:
            tag_value = tag_values['value']
            uuid = tag_values['uuid']
        except KeyError:
            tag_value = "Value Not Set Yet"
            uuid = "NO Value set"
        click.echo("{:30s} : {:35s} {}".format(str(tag_values['category_name']), str(tag_value), str(uuid)))
    click.echo()


@display.command(help="Print Tag Categories")
def categories():
    click.echo("\n{:31s} {}".format("Tag Categories", "Category UUID"))
    click.echo("-" * 150)
    for cats in tio.tags.list_categories():
        category_name = cats['name']
        category_uuid = cats['uuid']
        click.echo("{:31s} {}".format(str(category_name), str(category_uuid)))
    click.echo()


@display.command(help="Print SMTP information")
def smtp():
    try:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT server, port, from_email from smtp;")
            data = cur.fetchall()
            for settings in data:
                click.echo("\nYour email server: {}".format(settings[0]))
                click.echo("The email port is: {}".format(settings[1]))
                click.echo("Your email is: {}\n".format(settings[2]))
    except Error as E:
        click.echo("\nYou have no SMTP information saved.\n")
        click.echo("Error: ", E, "\n")


@display.command(help="Print Cloud information")
def cloud():
    click.echo("\n{:11s} {:15s} {:45s} {:40} {}".format("Source", "IP", "FQDN", "UUID", "First seen"))
    click.echo("-" * 150)
    for cloud_assets in tio.workbenches.assets(('sources', 'set-has', 'AWS'), ('sources', 'set-has', 'GCP'), ('sources', 'set-has', 'AZURE'), filter_type="or", age=90):
        for source in cloud_assets['sources']:
            if source['name'] != 'NESSUS_SCAN':
                asset_ip = cloud_assets['ipv4'][0]
                uuid = cloud_assets['id']
                try:
                    asset_fqdn = cloud_assets['fqdn'][0]
                except IndexError:
                    asset_fqdn = "NO FQDN found"

                click.echo("{:11s} {:15s} {:45s} {:40s} {}".format(str(source['name']), str(asset_ip),
                                                                   str(asset_fqdn), str(uuid),
                                                                   str(source['first_seen'])))
    click.echo()


@display.command(help="Print Network Information")
def networks():
    click.echo("\n{:45s} {:16} {}".format("Network Name", "# of Scanners", "UUID"))
    click.echo("-" * 150)
    for network in tio.networks.list():
        click.echo("{:45s} {:16} {}".format(str(network['name']), str(network['scanner_count']),
                                            str(network['uuid'])))
    click.echo()


@display.command(help="Print Navi Version")
def version():
    click.echo("\nCurrent Navi Version: {}\n".format(navi_version()))


@display.command(help="Print User group information")
@click.option('--membership', required=True, default='', help="Display Users that apart of a particular usergroup")
def usergroups(membership):

    if membership != '':
        click.echo("\n{:35s} {:40s} {:40s} {:10} {}".format("User Name", "Login email", "User UUID", "User ID",
                                                            "Enabled?"))
        click.echo("-" * 150)
        for user in tio.groups.list_users(membership):
            click.echo("{:35s} {:40s} {:40s} {:10} {}".format(str(user["name"]), str(user["username"]),
                                                              str(user['uuid']), str(user['id']),
                                                              str(user["enabled"])))
        click.echo()
    else:
        click.echo("\n{:35s} {:10s} {:40s} {}".format("Group Name", "Group ID", "Group UUID", "User Count"))
        click.echo("-" * 150)
        for ugroup in tio.groups.list():
            click.echo("{:35s} {:10s} {:40s} {}".format(str(ugroup['name']), str(ugroup['id']),
                                                    str(ugroup['uuid']), str(ugroup['user_count'])))
        click.echo()


@display.command(help="List All Credentials")
def credentials():
    try:

        click.echo("\n{:25s} {:25s} {:25s} {:25s} {:40s}".format("Credential Name", "Created By", "Credential Type", "Category", "Credential UUID"))
        click.echo("-" * 150)
        for cred in tio.credentials.list():
            creator = cred['created_by']['display_name']
            name = cred['name']
            type = cred['type']['name']
            cred_uuid = cred['uuid']
            category = cred['category']['name']
            click.echo("{:25s} {:25s} {:25s} {:25s} {:40s}".format(textwrap.shorten(name, width=25),
                                                                     textwrap.shorten(creator, width=25),
                                                                     textwrap.shorten(type, width=25),
                                                                     textwrap.shorten(category, width=25),
                                                                     textwrap.shorten(cred_uuid, width=40)))
        click.echo()
    except Exception as E:
        error_msg(E)



import click
import textwrap
from sqlite3 import Error
from .api_wrapper import tenb_connection
from .database import new_db_connection


def plugin_by_ip(ipaddr, plugin):
    try:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            try:
                cur = conn.cursor()
                cur.execute("SELECT output from vulns where asset_ip=\"%s\" and plugin_id=%s" % (ipaddr, plugin))
                rows = cur.fetchall()
                for plug in rows:
                    click.echo(plug[0])
            except:
                pass

    except Error as e:
        click.echo(e)

    except IndexError:
        click.echo("No information found for this plugin")


def vulns_by_uuid(uuid):
    try:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("select plugin_id, plugin_name, plugin_family, port, protocol, severity from vulns where asset_uuid='{}' and severity !='info';".format(uuid))

            data = cur.fetchall()
            click.echo("\n{:10s} {:80s} {:35s} {:6s} {:6s} {}".format("Plugin", "Plugin Name", "Plugin Family", "Port", "Proto", "Severity"))
            click.echo("-"*150)
            for vulns in data:
                plugin_id = vulns[0]
                plugin_name = vulns[1]
                plugin_family = vulns[2]
                port = vulns[3]
                protocol = vulns[4]
                severity = vulns[5]
                click.echo("{:10s} {:80s} {:35s} {:6s} {:6s} {}".format(plugin_id, textwrap.shorten(plugin_name, 80), textwrap.shorten(plugin_family, 35), port, protocol, severity))
            click.echo("")
    except Error as e:
        click.echo(e)


def info_by_uuid(uuid):
    try:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("select plugin_id, plugin_name, plugin_family, port, protocol, severity from vulns where asset_uuid='{}' and severity =='info';".format(uuid))

            data = cur.fetchall()
            click.echo("\n{:10s} {:90s} {:25s} {:6s} {:6s} {}".format("Plugin", "Plugin Name", "Plugin Family", "Port", "Proto", "Severity"))
            click.echo("-"*150)
            for vulns in data:
                plugin_id = vulns[0]
                plugin_name = vulns[1]
                plugin_family = vulns[2]
                port = vulns[3]
                protocol = vulns[4]
                severity = vulns[5]
                click.echo("{:10s} {:90s} {:25s} {:6s} {:6s} {}".format(plugin_id, plugin_name, plugin_family, port, protocol, severity))
            click.echo("")
    except Error as e:
        click.echo(e)


@click.command(help="Get IP specific information")
@click.argument('ipaddr')
@click.option('--plugin', default='', help='Find Details on a particular plugin ID')
@click.option('-n', is_flag=True, help='Netstat Established(58561) and Listening and Open Ports(14272)')
@click.option('-p', is_flag=True, help='Patch Information - 66334')
@click.option('-t', is_flag=True, help='Trace Route - 10287')
@click.option('-o', is_flag=True, help='Process Information - 70329')
@click.option('-c', is_flag=True, help='Connection Information - 64582')
@click.option('-s', is_flag=True, help='Services Running - 22964')
@click.option('-r', is_flag=True, help='Local Firewall Rules - 56310')
@click.option('-patches', is_flag=True, help='Missing Patches - 38153')
@click.option('-d', is_flag=True, help="Scan Detail: 19506 plugin output")
@click.option('-software', is_flag=True, help="Find software installed on Unix(22869) of windows(20811) hosts")
@click.option('-outbound', is_flag=True, help="outbound connections found by nnm")
@click.option('-exploit', is_flag=True, help="Display Solution, Description for each Exploit")
@click.option('-critical', is_flag=True, help="Display Plugin Output for each Critical Vuln")
@click.option('-details', is_flag=True, help="Details on an Asset: IP, UUID, Vulns, etc")
@click.option('-vulns', is_flag=True, help="Display all vulnerabilities and their plugin IDs")
@click.option('-info', is_flag=True, help="Display all info plugins and their IDs")
@click.pass_context
def ip(ctx, ipaddr, plugin, n, p, t, o, c, s, r, patches, d, software, outbound, exploit, critical, details, vulns, info):
    tio = tenb_connection()
    if d:
        click.echo('\nScan Detail')
        click.echo("-" * 15)
        click.echo()
        plugin_by_ip(ipaddr, str(19506))

    if n:
        click.echo("\nNetstat info")
        click.echo("Established and Listening")
        click.echo("-" * 15)
        click.echo()
        plugin_by_ip(ipaddr, str(58651))
        click.echo("\nNetstat Open Ports")
        click.echo("-" * 15)
        click.echo()
        plugin_by_ip(ipaddr, str(14272))

    if p:
        click.echo("\nPatch Information")
        click.echo("-" * 15)
        click.echo()
        plugin_by_ip(ipaddr, str(66334))

    if t:
        click.echo("\nTrace Route Info")
        click.echo("-" * 15)
        click.echo()
        plugin_by_ip(ipaddr, str(10287))

    if o:
        click.echo("\nProcess Info")
        click.echo("-" * 15)
        click.echo()
        plugin_by_ip(ipaddr, str(70329))
        plugin_by_ip(ipaddr, str(110483))

    if patches:
        click.echo("\nMissing Patches")
        click.echo("-" * 15)
        click.echo()
        plugin_by_ip(ipaddr, str(38153))
        plugin_by_ip(ipaddr, str(66334))

        click.echo("\nLast Reboot")
        click.echo("-" * 15)
        click.echo()
        plugin_by_ip(ipaddr, str(56468))

    if c:
        click.echo("\nConnection info")
        click.echo("-" * 15)
        click.echo()
        plugin_by_ip(ipaddr, str(64582))

    if s:
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                cur.execute("SELECT output, port from vulns where asset_ip=\"%s\" and plugin_id='22964'" % ipaddr)
                data = cur.fetchall()

                for plugins in data:
                    output = plugins[0]
                    port = plugins[1]
                    click.echo("\n{} {}".format(str(output), str(port)))
                click.echo()
        except IndexError:
            click.echo("No information for plugin 22964")

    if r:
        click.echo("Local Firewall Info")
        click.echo("-" * 15)
        plugin_by_ip(ipaddr, str(56310))
        plugin_by_ip(ipaddr, str(61797))

    if software:
        try:
            plugin_by_ip(ipaddr, str(22869))
            plugin_by_ip(ipaddr, str(20811))
        except IndexError:
            click.echo("No Software found")

    if outbound:
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                cur.execute("SELECT output, port, protocol from vulns where asset_ip=\"%s\" and plugin_id='16'" % ipaddr)

                data = cur.fetchall()
                click.echo("\n{:15s} {:5} {}".format("IP address", "Port", "Protocol"))
                click.echo("-" * 25)
                for plugins in data:
                    output = plugins[0]
                    port = plugins[1]
                    proto = plugins[2]
                    click.echo("\n{:15s} {:5} {}".format(str(output), str(port), str(proto)))
                click.echo()
        except Exception as E:
            click.echo("No information for plugin 16")
            click.echo(E)

    if exploit:
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                # Grab all of the UUIDs for the asset in question based on the IP provided
                cur.execute("SELECT uuid from assets where ip_address='" + ipaddr + "';")

                data = cur.fetchall()
                for assets in data:
                    asset_id = assets[0]

                    click.echo("\nExploitable Details for : {}\n".format(ipaddr))

                    vuln_data = tio.workbenches.asset_vulns(asset_id, ("plugin.attributes.exploit_available", "eq", "true"), age=90)

                    for plugins in vuln_data:
                        plugin = plugins['plugin_id']

                        plugin_data = tio.plugins.plugin_details(plugin)

                        click.echo("\n----Exploit Info----")
                        click.echo(plugin_data['name'])
                        click.echo()
                        for attribute in plugin_data['attributes']:

                            if attribute['attribute_name'] == 'cve':
                                cve = attribute['attribute_value']
                                click.echo("CVE ID : " + cve)

                            if attribute['attribute_name'] == 'description':
                                description = attribute['attribute_value']
                                click.echo("Description")
                                click.echo("------------\n")
                                click.echo(description)
                                click.echo()

                            if attribute['attribute_name'] == 'solution':
                                solution = attribute['attribute_value']
                                click.echo("\nSolution")
                                click.echo("------------\n")
                                click.echo(solution)
                                click.echo()
        except Exception as E:
            click.echo(E)

    if critical:
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                cur.execute("SELECT uuid from assets where ip_address='" + ipaddr + "';")

                data = cur.fetchall()
                for assets in data:
                    asset_id = assets[0]
                    click.echo("\nCritical Vulns for Ip Address : {}\n".format(ipaddr))

                    asset_vulns = tio.workbenches.asset_vulns(asset_id, age=90)

                    for severities in asset_vulns:
                        vuln_name = severities["plugin_name"]
                        plugin_id = severities["plugin_id"]
                        severity = severities["severity"]
                        state = severities["vulnerability_state"]

                        # only pull the critical vulns; critical = severity 4
                        if severity >= 4:
                            click.echo("Plugin Name : {}".format(vuln_name))
                            click.echo("ID : {}".format(str(plugin_id)))
                            click.echo("Severity : Critical")
                            click.echo("State : {}".format(state))
                            click.echo("----------------\n")
                            plugin_by_ip(str(ipaddr), str(plugin_id))
                            click.echo()
        except Exception as E:
            click.echo(E)

    if details:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT uuid from assets where ip_address='" + ipaddr + "';")

            data = cur.fetchall()
            for assets in data:
                asset_data = tio.workbenches.asset_info(str(assets[0]))

                try:
                    asset_id = asset_data['id']

                    click.echo("\nTenable ID")
                    click.echo("--------------")
                    click.echo(asset_id)

                    click.echo("\nNetwork Name")
                    click.echo("--------------")
                    click.echo(asset_data['network_name'])

                    click.echo("\nIdentities")
                    click.echo("--------------")
                    try:
                        for netbioss in asset_data['netbios_name']:
                            click.echo("Netbios - {}".format(netbioss))
                    except KeyError:
                        pass
                    try:
                        for fqdns in asset_data['fqdns']:
                            click.echo("FQDN - {}".format(fqdns))
                    except KeyError:
                        pass

                    try:
                        for hosts in asset_data['hostname']:
                            click.echo("Host Name - {}".format(hosts))
                    except KeyError:
                        pass

                    try:
                        for agentname in asset_data['agent_name']:
                            click.echo("Agent Name - {}".format(agentname))
                    except KeyError:
                        pass

                    try:
                        for awsid in asset_data['aws_ec2_instance_id']:
                            click.echo("AWS EC2 Instance ID - {}".format(awsid))
                    except KeyError:
                        pass

                    try:
                        for awsamiid in asset_data['aws_ec2_ami_id']:
                            click.echo("AWS EC2 AMI ID - {}".format(awsamiid))
                    except KeyError:
                        pass

                    try:
                        for awsname in asset_data['aws_ec2_name']:
                            click.echo("AWS EC2 Name - {}".format(awsname))
                    except KeyError:
                        pass

                    click.echo("\nOperating Systems")
                    click.echo("--------------")
                    try:
                        for oss in asset_data['operating_system']:
                            click.echo(oss)
                    except KeyError:
                        pass

                    try:
                        click.echo("\nIP Addresses:")
                        click.echo("--------------")
                        for ips in asset_data['ipv4']:
                            click.echo(ips)
                    except KeyError:
                        pass

                    try:
                        click.echo("\nMac Addresses:")
                        click.echo("--------------")
                        for macs in asset_data['mac_address']:
                            click.echo(macs)
                    except KeyError:
                        pass

                    try:
                        click.echo("\nCloud Information:")
                        click.echo("--------------")
                        for zone in asset_data['aws_availability_zone']:
                            click.echo("AWS Availability Zone - {}".format(zone))
                    except KeyError:
                        pass

                    try:
                        for groupname in asset_data['aws_ec2_instance_group_name']:
                            click.echo("AWS Instance group Name - {}".format(groupname))
                    except KeyError:
                        pass

                    try:
                        for zone in asset_data['aws_availability_zone']:
                            click.echo("AWS Availability Zone - {}".format(zone))
                    except KeyError:
                        pass
                    try:
                        for statename in asset_data['aws_ec2_instance_state_name']:
                            click.echo("AWS Instance State - {}".format(statename))
                    except KeyError:
                        pass
                    try:
                        for instatncetype in asset_data['aws_ec2_instance_type']:
                            click.echo("AWS Instance Type - {}".format(instatncetype))
                    except KeyError:
                        pass
                    try:
                        for region in asset_data['aws_region']:
                            click.echo("AWS Region - {}".format(region))
                    except KeyError:
                        pass

                    try:
                        for subnet in asset_data['aws_subnet_id']:
                            click.echo("AWS Subnet ID - {}".format(subnet))
                    except KeyError:
                        pass
                    try:
                        for vpc in asset_data['aws_vpc_id']:
                            click.echo("AWS VPC ID - {}".format(vpc))
                    except KeyError:
                        pass
                    try:
                        for azureid in asset_data['azure_resource_id']:
                            click.echo("Azure Resource ID - {}".format(azureid))
                    except KeyError:
                        pass
                    try:
                        for vmid in asset_data['azure_vm_id']:
                            click.echo("Azure VM ID - {}".format(vmid))
                    except KeyError:
                        pass

                    try:
                        for gcpid in asset_data['gcp_instance_id']:
                            click.echo("GCP Instance ID - {}".format(gcpid))
                    except KeyError:
                        pass
                    try:
                        for projectid in asset_data['gcp_project_id']:
                            click.echo("GCP Project ID- {}".format(projectid))
                    except KeyError:
                        pass
                    try:
                        for gcpzone in asset_data['gcp_zone']:
                            click.echo("GCP Zone - {}".format(gcpzone))
                    except KeyError:
                        pass
                    try:
                        click.echo("\nSources:")
                        click.echo("-" * 15)
                        for source in asset_data['sources']:
                            click.echo(source['name'])
                    except KeyError:
                        pass
                    try:
                        click.echo("\nTags:")
                        click.echo("-" * 15)
                        for tags in asset_data['tags']:
                            click.echo("{} : {}".format(tags["tag_key"], tags['tag_value']))
                    except KeyError:
                        pass

                    click.echo("\nVulnerability Counts")
                    click.echo("-" * 15)

                    asset_info = tio.workbenches.asset_info(asset_id)

                    for vuln in asset_info['counts']['vulnerabilities']['severities']:
                        click.echo("{} : {}".format(vuln["name"], vuln["count"]))

                    try:
                        click.echo("\nAsset Exposure Score : {}".format(asset_info['exposure_score']))
                        click.echo("\nAsset Criticality Score : {}".format(asset_info['acr_score']))
                    except KeyError:
                        pass

                    click.echo("\nLast Authenticated Scan Date - {}".format(asset_data['last_authenticated_scan_date']))
                    click.echo("\nLast Licensed Scan Date - {}".format(asset_data['last_licensed_scan_date']))
                    click.echo("-" * 50)
                    click.echo("-" * 50)
                except:
                    pass

    if vulns:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT uuid from assets where ip_address='" + ipaddr + "';")
            data = cur.fetchall()

            for assets in data:
                click.echo("\nAsset UUID: {}".format(assets[0]))
                click.echo("Asset IP: {}".format(ipaddr))
                click.echo("-" * 26)
                vulns_by_uuid(assets[0])

    if info:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT uuid from assets where ip_address='" + ipaddr + "';")
            data = cur.fetchall()

            for assets in data:
                click.echo("\nAsset UUID: {}".format(assets[0]))
                click.echo("Asset IP: {}".format(ipaddr))
                click.echo("-" * 26)
                info_by_uuid(assets[0])

    if plugin != '':
        plugin_by_ip(ipaddr, plugin)



import csv
import time
from .api_wrapper import tenb_connection

tio = tenb_connection()


def agent_export():

    with open('agent_data.csv', mode='w') as csv_file:
        agent_writer = csv.writer(csv_file, delimiter=',', quotechar='"')

        header_list = ["Agent Name", "IP Address", "Platform", "Last connected", "Last scanned", "Status", "Plugin Feed ID"]
        agent_writer.writerow(header_list)

        for agent in tio.agents.list(limit=5000):

            name = agent['name']
            ip = agent['ip']
            platform = agent['platform']
            plugin_feed = agent['plugin_feed_id']

            last_connect = agent['last_connect']
            connect_time = time.strftime("%a, %d %b %Y %H:%M:%S", time.localtime(last_connect))
            try:
                last_scanned = agent['last_scanned']
                scanned_time = time.strftime("%a, %d %b %Y %H:%M:%S", time.localtime(last_scanned))
            except KeyError:
                scanned_time = "Not Yet Scanned"
            status = agent['status']

            agent_writer.writerow([name, ip, platform, connect_time, scanned_time, status, plugin_feed])
    return



import click
from .api_wrapper import tenb_connection

tio = tenb_connection()


def nessus_scanners():
    click.echo("\n{:35s} {:20} {}".format("Scanner Name", "Scanner ID", "Scanner UUID"))
    click.echo("-" * 150)
    for scanners in tio.scanners.list():
        click.echo("{:35s} {:20} {}".format(str(scanners["name"]), str(scanners["id"]), str(scanners['uuid'])))
    click.echo()
